// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: helloworld.proto

package io.opentracing.contrib;

public interface HelloReplyOrBuilder extends
    // @@protoc_insertion_point(interface_extends:helloworld.HelloReply)
    com.google.protobuf.MessageOrBuilder {

  /**
   * <code>optional string message = 1;</code>
   */
  java.lang.String getMessage();
  /**
   * <code>optional string message = 1;</code>
   */
  com.google.protobuf.ByteString
      getMessageBytes();
}
