# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc
from grpc.framework.common import cardinality
from grpc.framework.interfaces.face import utilities as face_utilities

import command_line_pb2 as command__line__pb2


class CommandLineStub(object):

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.Echo = channel.unary_unary(
        '/command_line.CommandLine/Echo',
        request_serializer=command__line__pb2.CommandRequest.SerializeToString,
        response_deserializer=command__line__pb2.CommandResponse.FromString,
        )


class CommandLineServicer(object):

  def Echo(self, request, context):
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_CommandLineServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'Echo': grpc.unary_unary_rpc_method_handler(
          servicer.Echo,
          request_deserializer=command__line__pb2.CommandRequest.FromString,
          response_serializer=command__line__pb2.CommandResponse.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'command_line.CommandLine', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))
